# Copyright (C) 2003, 2004 Gregoire Lejeune <gregoire.lejeune@free.fr>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

require 'tempfile'
require 'mkmf'

require 'graphviz/node'
require 'graphviz/edge'
require 'graphviz/attrs'
require 'graphviz/constants'

class GraphViz 
  include Constants

  public

  ## Var: Output format (dot, png, jpeg, ...)
  @format
  ## Var: Output file name
  @filename
  ## Var: programm to use (dot|twopi)
  @prog

  ## Var: Graph name
  @name

  ## Var: defined attributs
  @graph
  @node
  @edge
  attr_accessor :node, :edge

@elements_order

  ##
  # In:
  #   - xNodeName : Name of the new node
  #   - *hOpt     : Node options
  # Out:
  #   - xNodeName as In
  #
  def add_node( xNodeName, *hOpt )
    @hoNodes[xNodeName] = GraphViz::Node::new( xNodeName )
   
    if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
        @hoNodes[xNodeName][xKey] = xValue
      end
    end

@elements_order.push( { 
  "type" => "node", 
  "name" => xNodeName,
  "value" => @hoNodes[xNodeName] 
} )
    
    return( @hoNodes[xNodeName] )
  end

  def add_edge( oNodeOne, oNodeTwo, *hOpt )
    oEdge = GraphViz::Edge::new( oNodeOne, oNodeTwo )
    
    if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
        oEdge[xKey] = xValue
      end
    end

@elements_order.push( { 
  "type" => "edge", 
  "value" => oEdge
} )
    
    @loEdges.push( oEdge )
    return( oEdge )
  end

  def add_graph( xGraphName, *hOpt )
    @hoGraphs[xGraphName] = GraphViz::new( xGraphName, "parent" => self, "type" => @oGraphType )
   
    if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
        @hoGraphs[xGraphName][xKey] = xValue
      end
    end

@elements_order.push( { 
  "type" => "graph", 
  "name" => xGraphName,
  "value" => @hoGraphs[xGraphName] 
} )
    
    return( @hoGraphs[xGraphName] )
  end

  def []=( xAttrName, xValue )
    @graph[xAttrName] = xValue
  end

  def []( xAttrName )
    return( @graph[xAttrName].clone )
  end
  
  def output( *hOpt )
    xDOTScript = ""
    xLastType = nil
    xSeparator = ""
    xData = ""

    @elements_order.each { |kElement|
      if xLastType.nil? == true or xLastType != kElement["type"]
        
        if xData.length > 0 
          case xLastType
            when "graph_attr"
              xDOTScript << "  " + xData + ";\n"
  
            when "node_attr"
              xDOTScript << "  node [" + xData + "];\n"
            
            when "edge_attr"
              xDOTScript << "  edge [" + xData + "];\n"
          end
        end
        
        xSeparator = ""
        xData = ""
      end

      xLastType = kElement["type"]

      case kElement["type"]
        when "graph_attr"
          xData << xSeparator + kElement["name"] + " = \"" + kElement["value"] + "\""
          xSeparator = "; "

        when "node_attr"
          xData << xSeparator + kElement["name"] + " = \"" + kElement["value"] + "\""
          xSeparator = ", "

        when "edge_attr"
          xData << xSeparator + kElement["name"] + " = \"" + kElement["value"] + "\""
          xSeparator = ", "

        when "node"
          xDOTScript << "  " + kElement["value"].output() + "\n"

        when "edge"
          xDOTScript << "  " + kElement["value"].output( @oGraphType ) + "\n"

        when "graph"
          xDOTScript << kElement["value"].output() + "\n"

        else
          raise ArgumentError, "Don't know what to do with element type '#{kElement['type']}'"
      end
    }
    
	xDOTScript << "}"

    if @oParentGraph.nil? == false
      xDOTScript = "subgraph #{@name} {\n" << xDOTScript

      return( xDOTScript )
    else
      if hOpt.nil? == false and hOpt[0].nil? == false
        hOpt[0].each do |xKey, xValue|
          case xKey
            when "output"
              if FORMATS.index( xValue ).nil? == true
                raise ArgumentError, "output format '#{xValue}' invalid"
              end
              @format = xValue
            when "file"
              @filename = xValue
            when "use"
              if PROGRAMS.index( xValue ).nil? == true
                raise ArgumentError, "can't use '#{xValue}'"
              end
              @prog = xValue
            else
              raise ArgumentError, "option #{xKey} unknown"
          end
        end
      end
  
      xDOTScript = "#{@oGraphType} #{@name} {\n" << xDOTScript

      ## Act: Save script and send it to dot
      t = Tempfile::open( File.basename($0) + "." )
      t.print( xDOTScript )
      t.close
      
      #cmd = find_executable( @prog )
      cmd = find_executable0( @prog )
      if cmd == nil
        raise StandardError, "GraphViz not installed"
      end
      
      xFile = ""
      xFile = "-o #{@filename}" if @filename.nil? == false
      xCmd = "#{cmd} -T#{@format} #{xFile} #{t.path}"
      
      f = IO.popen( xCmd )
      print f.readlines
      f.close
    end
  end
  
  def output_OLD( *hOpt )
    xDOTScript = ""

    ## Add default attributs' graph values
    graph = ""
    separator = ""
    @graph.data.each do |k,v|
      graph << separator + k + " = \"" + v + "\""
      separator = "; "
    end
    if graph.length > 0 
      xDOTScript << "  " + graph + ";\n"
    end

    ## Add default attributs' node values
    node = ""
    separator = ""
    @node.data.each do |k,v|
      node << separator + k + " = \"" + v + "\""
      separator = ", "
    end
    if node.length > 0 
      xDOTScript << "  node [" + node + "];\n"
    end

    ## Add default attributs' edge values
    edge = ""
    separator = ""
    @edge.data.each do |k,v|
      edge << separator + k + " = \"" + v + "\""
      separator = ", "
    end
    if edge.length > 0 
      xDOTScript << "  edge [" + edge + "];\n"
    end

    # graphs
    @hoGraphs.each do |xName, oGraph|
      xDOTScript << oGraph.output() + "\n"
    end
    
    # nodes
    @hoNodes.each do |xName, oNode|
      xDOTScript << "  " + oNode.output() + "\n"
    end
    
    # edges
    @loEdges.each do |oEdge|
      xDOTScript << "  " + oEdge.output() + "\n"
    end

    xDOTScript << "}"
      
    if @oParentGraph.nil? == false
      xDOTScript = "subgraph #{@name} {\n" << xDOTScript

      return( xDOTScript )
    else
      if hOpt.nil? == false and hOpt[0].nil? == false
        hOpt[0].each do |xKey, xValue|
          case xKey
            when "output"
              if FORMATS.index( xValue ).nil? == true
                raise ArgumentError, "output format '#{xValue}' invalid"
              end
              @format = xValue
            when "file"
              @filename = xValue
            when "use"
              if PROGRAMS.index( xValue ).nil? == true
                raise ArgumentError, "can't use '#{xValue}'"
              end
              @prog = xValue
            else
              raise ArgumentError, "option #{xKey} unknown"
          end
        end
      end
  
      xDOTScript = "#{@oGraphType} #{@name} {\n" << xDOTScript

      ## Act: Save script and send it to dot
      t = Tempfile::open( File.basename($0) + "." )
      t.print( xDOTScript )
      t.close
      
      #cmd = find_executable( @prog )
      cmd = find_executable0( @prog )
      if cmd == nil
        raise StandardError, "GraphViz not installed"
      end
      
      xFile = ""
      xFile = "-o #{@filename}" if @filename.nil? == false
      xCmd = "#{cmd} -T#{@format} #{xFile} #{t.path}"
      
      f = IO.popen( xCmd )
      print f.readlines
      f.close
    end
  end

  def set_position( xType, xKey, xValue )
@elements_order.push( { 
  "type" => "#{xType}_attr", 
  "name" => xKey,
  "value" => xValue 
} )
  end
  
## ----------------------------------------------------------------------------

  private 

  ## Var: Nodes, Edges and Graphs tables
  @hoNodes
  @loEdges
  @hoGraphs

  ## Var: Parent graph
  @oParentGraph

  ## Var: Type de graphe (orient ou non)
  @oGraphType
  
  def initialize( xGraphName, *hOpt )
    @format   = "dot"
    @filename = nil
    @prog     = "dot"
    @name = xGraphName

@elements_order = Array::new()

    @oParentGraph = nil
	@oGraphType = "digraph"
    
    @hoNodes  = Hash::new()
    @loEdges  = Array::new()
    @hoGraphs = Hash::new()
    
    @node  = GraphViz::Attrs::new( self, "node",  NODESATTRS  )
    @edge  = GraphViz::Attrs::new( self, "edge",  EDGESATTRS  )
    @graph = GraphViz::Attrs::new( self, "graph", GRAPHSATTRS )

    if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
        case xKey
          when "output"
            if FORMATS.index( xValue ).nil? == true
              raise ArgumentError, "output format '#{xValue}' invalid"
            end
            @format = xValue
          when "use"
            if PROGRAMS.index( xValue ).nil? == true
              raise ArgumentError, "can't use '#{xValue}'"
            end
            @prog = xValue
          when "file"
            @filename = xValue
          when "parent"
            @oParentGraph = xValue
		  when "type"
		    if GRAPHTYPE.index( xValue ).nil? == true
			  raise ArgumentError, "graph type '#{xValue}' unknow"
			end
		    @oGraphType = xValue
          else
            self[xKey] = xValue
        end
      end
    end
  end
  
#  def find_executable(bin, path = nil)
#    path = (path || ENV['PATH']).split(File::PATH_SEPARATOR)
#    file = nil
#    path.each do |dir|
#      return file if File.executable?(file = File.join(dir, bin))
#    end
#    nil
#  end
end

